﻿using System;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Text;
using System.Web;
using System.Web.Http.Filters;
using Kreta.Core;
using Kreta.Core.Domain;
using Kreta.Core.Enum;
using Kreta.Core.Exceptions;
using Kreta.Core.Validation.Exceptions;
using Kreta.Core.Validation.Extensions;
using Kreta.Framework.Entities;
using Kreta.Naplo.Dto.V3.Exception;
using Kreta.Naplo.WebApi.V3.Common.Logic;
using Kreta.Web.Logging.Extensions;
using Kreta.Web.Logging.Logger;
using Newtonsoft.Json;

namespace Kreta.Naplo.WebApi.FilterAttributes
{
    internal sealed class DefaultExceptionFilterAttribute : ExceptionFilterAttribute
    {
        private readonly string _mobileUserLogKey = "MobileUser";

        public override void OnException(HttpActionExecutedContext actionExecutedContext)
        {
            if (actionExecutedContext.Request.RequestUri.AbsoluteUri.ToLower().Contains("v2"))
            {
                HandleV2Exeption(actionExecutedContext);
            }
            else
            {
                HandleException(actionExecutedContext);
            }


        }

        private void HandleException(HttpActionExecutedContext actionExecutedContext)
        {
            if (actionExecutedContext.Exception != null)
            {
                NaploExceptionResponseDto response;
                if ((actionExecutedContext.Exception.InnerException ?? actionExecutedContext.Exception) is ValidationException validationException)
                {
                    const BlExceptionType ResponseExceptionType = BlExceptionType.ModelValidacio;
                    var errorList = validationException.ValidationItems.Select(x => new DetailedErrorItem(x.Id, x.Message, BlExceptionType.ModelValidacio)).ToList();
                    response = new NaploExceptionResponseDto(ResponseExceptionType, ResponseExceptionType.ToDisplayName(), errorList);
                    actionExecutedContext.Response = actionExecutedContext.Request.CreateResponse(HttpStatusCode.BadRequest);
                    actionExecutedContext.Response.Content = SerializeToHttpContent(response);
                }
                else if ((actionExecutedContext.Exception.InnerException ?? actionExecutedContext.Exception) is EntityNotFoundException)
                {
                    const BlExceptionType ResponseExceptionType = BlExceptionType.NemLetezoEntitas;

                    response = new NaploExceptionResponseDto(ResponseExceptionType, ResponseExceptionType.ToDisplayName());
                    actionExecutedContext.Response = actionExecutedContext.Request.CreateResponse(HttpStatusCode.NotFound);
                    actionExecutedContext.Response.Content = SerializeToHttpContent(response);
                }
                else if ((actionExecutedContext.Exception.InnerException ?? actionExecutedContext.Exception) is BlException blException)
                {
                    var responseExceptionType = blException.ExceptionType;
                    var responseStatusCode = blException.GetResponseStatusCode();

                    if (responseStatusCode == HttpStatusCode.Forbidden && string.IsNullOrEmpty(blException.Message))
                    {
                        actionExecutedContext.Response = actionExecutedContext.Request.CreateResponse(responseStatusCode);
                        actionExecutedContext.Response.Content = new StringContent("permission_denied");
                        return;
                    }

                    response = new NaploExceptionResponseDto(responseExceptionType,
                        blException.Message,
                        blException.ErrorList);
                    actionExecutedContext.Response = actionExecutedContext.Request.CreateResponse(responseStatusCode);
                    actionExecutedContext.Response.Content = SerializeToHttpContent(response);
                }
                else if ((actionExecutedContext.Exception.InnerException ?? actionExecutedContext.Exception) is OperationCanceledException)
                {
                    var taskCancelledException = new BlException(BlExceptionType.TaskCancelled.ToDisplayName(), BlExceptionType.TaskCancelled, actionExecutedContext.Exception);

                    response = new NaploExceptionResponseDto(taskCancelledException.ExceptionType, taskCancelledException.ExceptionType.ToDisplayName());
                    actionExecutedContext.Response = actionExecutedContext.Request.CreateResponse(taskCancelledException.GetResponseStatusCode());
                    actionExecutedContext.Response.Content = SerializeToHttpContent(response);
                    actionExecutedContext.Exception = taskCancelledException;
                }
                else
                {
                    response = new NaploExceptionResponseDto(BlExceptionType.None, BlExceptionType.None.ToDisplayName());
                    actionExecutedContext.Response = actionExecutedContext.Request.CreateResponse(HttpStatusCode.InternalServerError);
                    actionExecutedContext.Response.Content = SerializeToHttpContent(response);
                }

                actionExecutedContext.Exception.Data.Add(_mobileUserLogKey, JsonConvert.SerializeObject(FelhasznaloLogic.GetFelhasznalo()));
                if (!actionExecutedContext.Exception.Data.Contains(RequestResponseLoggingFields.Server.ExceptionId))
                {
                    actionExecutedContext.Exception.Data.Add(RequestResponseLoggingFields.Server.ExceptionId, response.ExceptionId);
                }
                HttpContext.Current.AddException(actionExecutedContext.Exception);
            }
        }

        private static void HandleV2Exeption(HttpActionExecutedContext actionExecutedContext)
        {
            if (actionExecutedContext.Exception != null)
            {
                if (actionExecutedContext.Exception is ValidationException validationException)
                {
                    actionExecutedContext.Response = actionExecutedContext.Request.CreateResponse(validationException.GetHttpStatusCode(), validationException);
                }
                else
                {
                    Guid exceptionGuid = Guid.NewGuid();

                    actionExecutedContext.Response = actionExecutedContext.Request.CreateResponse(HttpStatusCode.InternalServerError);

                    actionExecutedContext.Response.Content = SerializeToHttpContent(new { IsException = true, ExceptionId = exceptionGuid, Type = actionExecutedContext.Exception.GetType().Name, actionExecutedContext.Exception.Message });

                    if (!actionExecutedContext.Exception.Data.Contains(RequestResponseLoggingFields.Server.ExceptionId))
                    {
                        actionExecutedContext.Exception.Data.Add(RequestResponseLoggingFields.Server.ExceptionId, exceptionGuid);
                    }

                    HttpContext.Current.AddException(actionExecutedContext.Exception);
                }
            }
        }

        private static HttpContent SerializeToHttpContent(object model) => new StringContent(JsonConvert.SerializeObject(model), Encoding.UTF8, "application/json");
    }
}
