﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Security.Claims;
using System.Security.Principal;
using System.Threading;
using System.Threading.Tasks;
using System.Web.Http.Controllers;
using System.Web.Http.Filters;
using Kreta.Naplo.Domain.V3.Enum;
using Kreta.Naplo.WebApi.V3.Common.Logic;

namespace Kreta.Naplo.WebApi.FilterAttributes
{
    public sealed class IdpAuthorizeAttribute : AuthorizationFilterAttribute, IOverrideFilter
    {
        private readonly IEnumerable<FelhasznaloSzerepkor> _permitForRoles;

        public Type FiltersToOverride => typeof(IAuthorizationFilter);

        public IdpAuthorizeAttribute(params FelhasznaloSzerepkor[] permitForRoles)
        {
            _permitForRoles = permitForRoles;
        }

        public override Task OnAuthorizationAsync(HttpActionContext actionContext, CancellationToken cancellationToken)
        {
            if (actionContext == null)
            {
                throw new ArgumentNullException(nameof(actionContext));
            }

            var principal = actionContext.ControllerContext.RequestContext.Principal as ClaimsPrincipal;

            if (!(principal?.Identity?.IsAuthenticated ?? false))
            {
                actionContext.Response = new HttpResponseMessage(HttpStatusCode.Unauthorized)
                {
                    Content = new StringContent("invalid_grant")
                };
                return Task.FromResult(0);
            }

            actionContext.ControllerContext.RequestContext.Principal = new ClaimsPrincipal(new ClaimsIdentity(GetCustomIdentity(), principal.Claims));

            if (_permitForRoles == null || !_permitForRoles.Any())
            {
                return Task.FromResult(0);
            }

            if (HasRole(principal))
            {
                return Task.FromResult(0);
            }

            actionContext.Response = new HttpResponseMessage(HttpStatusCode.Forbidden)
            {
                Content = new StringContent("permission_denied")
            };
            return Task.FromResult(0);
        }

        private bool HasRole(IPrincipal principal)
        {
            if (principal is ClaimsPrincipal claimsPrincipal)
            {
                var roleClaims = claimsPrincipal.Claims.Where(claim => claim.Type == ClaimTypes.Role);

                foreach (var roleClaim in roleClaims)
                {
                    if (Enum.TryParse(roleClaim.Value, out FelhasznaloSzerepkor role) && _permitForRoles.Contains(role))
                    {
                        return true;
                    }
                }
            }

            return false;
        }

        private static GenericIdentity GetCustomIdentity()
        {
            var user = FelhasznaloLogic.GetFelhasznalo();
            return new GenericIdentity($"{user.UserIdpUniqueId}/{user.InstituteCode}/{user.UserId}/{user.SchoolYearId}");
        }
    }
}
